package org.lwjgl.opengl;

import java.awt.*;
import java.awt.event.*;
import org.lwjgl.*;

public class AWTGLCanvas extends Canvas implements Drawable, ComponentListener, HierarchyListener
{
    private static final long serialVersionUID = 1L;
    private static final AWTCanvasImplementation implementation;
    private boolean update_context;
    private Object SYNC_LOCK;
    private final PixelFormat pixel_format;
    private final Drawable drawable;
    private final ContextAttribs attribs;
    private PeerInfo peer_info;
    private Context context;
    private int reentry_count;
    private boolean first_run;
    
    static AWTCanvasImplementation createImplementation() {
        switch (LWJGLUtil.getPlatform()) {
            case 1: {
                return new LinuxCanvasImplementation();
            }
            case 3: {
                return new WindowsCanvasImplementation();
            }
            case 2: {
                return new MacOSXCanvasImplementation();
            }
            default: {
                throw new IllegalStateException("Unsupported platform");
            }
        }
    }
    
    private void setUpdate() {
        synchronized (this.SYNC_LOCK) {
            this.update_context = true;
        }
    }
    
    public Context getContext() {
        return this.context;
    }
    
    public AWTGLCanvas() throws LWJGLException {
        this(new PixelFormat());
    }
    
    public AWTGLCanvas(final PixelFormat pixel_format) throws LWJGLException {
        this(GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice(), pixel_format);
    }
    
    public AWTGLCanvas(final GraphicsDevice device, final PixelFormat pixel_format) throws LWJGLException {
        this(device, pixel_format, null);
    }
    
    public AWTGLCanvas(final GraphicsDevice device, final PixelFormat pixel_format, final Drawable drawable) throws LWJGLException {
        this(device, pixel_format, drawable, null);
    }
    
    public AWTGLCanvas(final GraphicsDevice device, final PixelFormat pixel_format, final Drawable drawable, final ContextAttribs attribs) throws LWJGLException {
        super(AWTGLCanvas.implementation.findConfiguration(device, pixel_format));
        this.SYNC_LOCK = new Object();
        if (pixel_format == null) {
            throw new NullPointerException("Pixel format must be non-null");
        }
        this.addHierarchyListener(this);
        this.addComponentListener(this);
        this.drawable = drawable;
        this.pixel_format = pixel_format;
        this.attribs = attribs;
    }
    
    public void addNotify() {
        super.addNotify();
    }
    
    public void removeNotify() {
        synchronized (this.SYNC_LOCK) {
            this.destroyContext();
            super.removeNotify();
        }
    }
    
    public void setSwapInterval(final int swap_interval) {
        synchronized (this.SYNC_LOCK) {
            if (this.context == null) {
                throw new IllegalStateException("Canvas not yet displayable");
            }
            Context.setSwapInterval(swap_interval);
        }
    }
    
    public void setVSyncEnabled(final boolean enabled) {
        this.setSwapInterval(enabled ? 1 : 0);
    }
    
    public void swapBuffers() throws LWJGLException {
        synchronized (this.SYNC_LOCK) {
            if (this.context == null) {
                throw new IllegalStateException("Canvas not yet displayable");
            }
            Context.swapBuffers();
        }
    }
    
    public void releaseContext() throws LWJGLException {
        synchronized (this.SYNC_LOCK) {
            if (this.context == null) {
                throw new IllegalStateException("Canvas not yet displayable");
            }
            if (this.context.isCurrent()) {
                Context.releaseCurrentContext();
            }
        }
    }
    
    public void makeCurrent() throws LWJGLException {
        synchronized (this.SYNC_LOCK) {
            if (this.context == null) {
                throw new IllegalStateException("Canvas not yet displayable");
            }
            this.context.makeCurrent();
        }
    }
    
    private void destroyContext() {
        synchronized (this.SYNC_LOCK) {
            try {
                if (this.context != null) {
                    this.context.forceDestroy();
                    this.context = null;
                    this.reentry_count = 0;
                    this.peer_info.destroy();
                    this.peer_info = null;
                }
            }
            catch (LWJGLException e) {
                throw new RuntimeException(e);
            }
        }
    }
    
    protected void initGL() {
    }
    
    protected void paintGL() {
    }
    
    public final void paint(final Graphics g) {
        LWJGLException exception = null;
        synchronized (this.SYNC_LOCK) {
            if (!this.isDisplayable()) {
                return;
            }
            try {
                if (this.peer_info == null) {
                    this.peer_info = AWTGLCanvas.implementation.createPeerInfo(this, this.pixel_format);
                }
                this.peer_info.lockAndGetHandle();
                try {
                    if (this.context == null) {
                        this.context = new Context(this.peer_info, this.attribs, (this.drawable != null) ? this.drawable.getContext() : null);
                        this.first_run = true;
                    }
                    if (this.reentry_count == 0) {
                        this.context.makeCurrent();
                    }
                    ++this.reentry_count;
                    try {
                        if (this.update_context) {
                            this.context.update();
                            this.update_context = false;
                        }
                        if (this.first_run) {
                            this.first_run = false;
                            this.initGL();
                        }
                        this.paintGL();
                    }
                    finally {
                        --this.reentry_count;
                        if (this.reentry_count == 0) {
                            Context.releaseCurrentContext();
                        }
                    }
                }
                finally {
                    this.peer_info.unlock();
                }
            }
            catch (LWJGLException e) {
                exception = e;
            }
        }
        if (exception != null) {
            this.exceptionOccurred(exception);
        }
    }
    
    protected void exceptionOccurred(final LWJGLException exception) {
        LWJGLUtil.log("Unhandled exception occurred, skipping paint(): " + exception);
    }
    
    public void update(final Graphics g) {
        this.paint(g);
    }
    
    public void componentShown(final ComponentEvent e) {
    }
    
    public void componentHidden(final ComponentEvent e) {
    }
    
    public void componentResized(final ComponentEvent e) {
        this.setUpdate();
    }
    
    public void componentMoved(final ComponentEvent e) {
        this.setUpdate();
    }
    
    public void setLocation(final int x, final int y) {
        super.setLocation(x, y);
        this.setUpdate();
    }
    
    public void setLocation(final Point p) {
        super.setLocation(p);
        this.setUpdate();
    }
    
    public void setSize(final Dimension d) {
        super.setSize(d);
        this.setUpdate();
    }
    
    public void setSize(final int width, final int height) {
        super.setSize(width, height);
        this.setUpdate();
    }
    
    public void setBounds(final int x, final int y, final int width, final int height) {
        super.setBounds(x, y, width, height);
        this.setUpdate();
    }
    
    public void hierarchyChanged(final HierarchyEvent e) {
        this.setUpdate();
    }
    
    static {
        Sys.initialize();
        implementation = createImplementation();
    }
}
