package org.lwjgl.opengl;

import java.nio.*;
import org.lwjgl.*;

final class Context
{
    private static final ContextImplementation implementation;
    private static final ThreadLocal current_context_local;
    private final ByteBuffer handle;
    private final PeerInfo peer_info;
    private final IntBuffer attribList;
    private final boolean forwardCompatible;
    private boolean destroyed;
    private boolean destroy_requested;
    private Thread thread;
    
    private static ContextImplementation createImplementation() {
        switch (LWJGLUtil.getPlatform()) {
            case 1: {
                return new LinuxContextImplementation();
            }
            case 3: {
                return new WindowsContextImplementation();
            }
            case 2: {
                return new MacOSXContextImplementation();
            }
            default: {
                throw new IllegalStateException("Unsupported platform");
            }
        }
    }
    
    PeerInfo getPeerInfo() {
        return this.peer_info;
    }
    
    static Context getCurrentContext() {
        return Context.current_context_local.get();
    }
    
    Context(final PeerInfo peer_info, final ContextAttribs attribs, final Context shared_context) throws LWJGLException {
        final Context context_lock = (shared_context != null) ? shared_context : this;
        synchronized (context_lock) {
            if (shared_context != null && shared_context.destroyed) {
                throw new IllegalArgumentException("Shared context is destroyed");
            }
            GLContext.loadOpenGLLibrary();
            try {
                this.peer_info = peer_info;
                if (attribs != null) {
                    this.attribList = attribs.getAttribList();
                    this.forwardCompatible = attribs.isForwardCompatible();
                }
                else {
                    this.attribList = null;
                    this.forwardCompatible = false;
                }
                this.handle = Context.implementation.create(peer_info, this.attribList, (shared_context != null) ? shared_context.handle : null);
            }
            catch (LWJGLException e) {
                GLContext.unloadOpenGLLibrary();
                throw e;
            }
        }
    }
    
    public static void releaseCurrentContext() throws LWJGLException {
        final Context current_context = getCurrentContext();
        if (current_context != null) {
            Context.implementation.releaseCurrentContext();
            GLContext.useContext(null);
            Context.current_context_local.set(null);
            synchronized (current_context) {
                current_context.thread = null;
                current_context.checkDestroy();
            }
        }
    }
    
    public synchronized void releaseDrawable() throws LWJGLException {
        if (this.destroyed) {
            throw new IllegalStateException("Context is destroyed");
        }
        Context.implementation.releaseDrawable(this.getHandle());
    }
    
    public synchronized void update() {
        if (this.destroyed) {
            throw new IllegalStateException("Context is destroyed");
        }
        Context.implementation.update(this.getHandle());
    }
    
    public static void swapBuffers() throws LWJGLException {
        Context.implementation.swapBuffers();
    }
    
    private boolean canAccess() {
        return this.thread == null || Thread.currentThread() == this.thread;
    }
    
    private void checkAccess() {
        if (!this.canAccess()) {
            throw new IllegalStateException("From thread " + Thread.currentThread() + ": " + this.thread + " already has the context current");
        }
    }
    
    public synchronized void makeCurrent() throws LWJGLException {
        this.checkAccess();
        if (this.destroyed) {
            throw new IllegalStateException("Context is destroyed");
        }
        this.thread = Thread.currentThread();
        Context.current_context_local.set(this);
        Context.implementation.makeCurrent(this.peer_info, this.handle);
        GLContext.useContext(this, this.forwardCompatible);
    }
    
    ByteBuffer getHandle() {
        return this.handle;
    }
    
    public synchronized boolean isCurrent() throws LWJGLException {
        if (this.destroyed) {
            throw new IllegalStateException("Context is destroyed");
        }
        return Context.implementation.isCurrent(this.handle);
    }
    
    private void checkDestroy() {
        if (!this.destroyed && this.destroy_requested) {
            try {
                this.releaseDrawable();
                Context.implementation.destroy(this.peer_info, this.handle);
                this.destroyed = true;
                this.thread = null;
                GLContext.unloadOpenGLLibrary();
            }
            catch (LWJGLException e) {
                LWJGLUtil.log("Exception occurred while destroying context: " + e);
            }
        }
    }
    
    public static void setSwapInterval(final int value) {
        Context.implementation.setSwapInterval(value);
    }
    
    public synchronized void forceDestroy() throws LWJGLException {
        this.checkAccess();
        this.destroy();
    }
    
    public synchronized void destroy() throws LWJGLException {
        if (this.destroyed) {
            return;
        }
        this.destroy_requested = true;
        final boolean was_current = this.isCurrent();
        int error = 0;
        if (was_current) {
            if (GLContext.getCapabilities() != null && GLContext.getCapabilities().OpenGL11) {
                error = GL11.glGetError();
            }
            releaseCurrentContext();
        }
        this.checkDestroy();
        if (was_current && error != 0) {
            throw new OpenGLException(error);
        }
    }
    
    static {
        current_context_local = new ThreadLocal();
        Sys.initialize();
        implementation = createImplementation();
    }
}
