package org.lwjgl.opengl;

import java.nio.*;
import java.security.*;
import java.lang.reflect.*;
import java.util.*;
import org.lwjgl.*;

public final class GLContext
{
    private static final ThreadLocal current_capabilities;
    private static CapabilitiesCacheEntry fast_path_cache;
    private static final ThreadLocal thread_cache_entries;
    private static final Map capability_cache;
    private static int gl_ref_count;
    private static boolean did_auto_load;
    
    public static ContextCapabilities getCapabilities() {
        final CapabilitiesCacheEntry recent_cache_entry = GLContext.fast_path_cache;
        if (recent_cache_entry.owner == Thread.currentThread()) {
            return recent_cache_entry.capabilities;
        }
        return getThreadLocalCapabilities();
    }
    
    private static ContextCapabilities getThreadLocalCapabilities() {
        return GLContext.current_capabilities.get();
    }
    
    static void setCapabilities(final ContextCapabilities capabilities) {
        GLContext.current_capabilities.set(capabilities);
        CapabilitiesCacheEntry thread_cache_entry = GLContext.thread_cache_entries.get();
        if (thread_cache_entry == null) {
            thread_cache_entry = new CapabilitiesCacheEntry();
            GLContext.thread_cache_entries.set(thread_cache_entry);
        }
        thread_cache_entry.owner = Thread.currentThread();
        thread_cache_entry.capabilities = capabilities;
        GLContext.fast_path_cache = thread_cache_entry;
    }
    
    static long getPlatformSpecificFunctionAddress(final String function_prefix, final String[] os_prefixes, final String[] os_function_prefixes, final String function) {
        final String os_name = AccessController.doPrivileged((PrivilegedAction<String>)new PrivilegedAction() {
            public Object run() {
                return System.getProperty("os.name");
            }
        });
        for (int i = 0; i < os_prefixes.length; ++i) {
            if (os_name.startsWith(os_prefixes[i])) {
                final String platform_function_name = function.replaceFirst(function_prefix, os_function_prefixes[i]);
                final long address = getFunctionAddress(platform_function_name);
                return address;
            }
        }
        return 0L;
    }
    
    static native long getFunctionAddress(final String p0);
    
    static int getSupportedExtensions(final Set supported_extensions) {
        final String version = GL11.glGetString(7938);
        if (version == null) {
            throw new IllegalStateException("glGetString(GL_VERSION) returned null - possibly caused by missing current context.");
        }
        final StringTokenizer version_tokenizer = new StringTokenizer(version, ". ");
        final String major_string = version_tokenizer.nextToken();
        final String minor_string = version_tokenizer.nextToken();
        int majorVersion = 0;
        int minorVersion = 0;
        try {
            majorVersion = Integer.parseInt(major_string);
            minorVersion = Integer.parseInt(minor_string);
        }
        catch (NumberFormatException e) {
            LWJGLUtil.log("The major and/or minor OpenGL version is malformed: " + e.getMessage());
        }
        if (4 <= majorVersion) {
            supported_extensions.add("OpenGL40");
        }
        if (3 < majorVersion || (3 == majorVersion && 3 <= minorVersion)) {
            supported_extensions.add("OpenGL33");
        }
        if (3 < majorVersion || (3 == majorVersion && 2 <= minorVersion)) {
            supported_extensions.add("OpenGL32");
        }
        if (3 < majorVersion || (3 == majorVersion && 1 <= minorVersion)) {
            supported_extensions.add("OpenGL31");
        }
        if (3 <= majorVersion) {
            supported_extensions.add("OpenGL30");
        }
        if (2 < majorVersion || (2 == majorVersion && 1 <= minorVersion)) {
            supported_extensions.add("OpenGL21");
        }
        if (2 <= majorVersion) {
            supported_extensions.add("OpenGL20");
        }
        if (1 < majorVersion || 5 <= minorVersion) {
            supported_extensions.add("OpenGL15");
        }
        if (1 < majorVersion || 4 <= minorVersion) {
            supported_extensions.add("OpenGL14");
        }
        if (1 < majorVersion || 3 <= minorVersion) {
            supported_extensions.add("OpenGL13");
        }
        if (1 < majorVersion || 2 <= minorVersion) {
            supported_extensions.add("OpenGL12");
        }
        if (1 < majorVersion || 1 <= minorVersion) {
            supported_extensions.add("OpenGL11");
        }
        int profileMask = 0;
        if (majorVersion < 3) {
            final String extensions_string = GL11.glGetString(7939);
            if (extensions_string == null) {
                throw new IllegalStateException("glGetString(GL_EXTENSIONS) returned null - is there a context current?");
            }
            final StringTokenizer tokenizer = new StringTokenizer(extensions_string);
            while (tokenizer.hasMoreTokens()) {
                supported_extensions.add(tokenizer.nextToken());
            }
        }
        else {
            final IntBuffer buffer = BufferUtils.createIntBuffer(16);
            GL11.glGetInteger(33309, buffer);
            for (int extensionCount = buffer.get(0), i = 0; i < extensionCount; ++i) {
                supported_extensions.add(GL30.glGetStringi(7939, i));
            }
            if (3 < majorVersion || 2 <= minorVersion) {
                Util.checkGLError();
                GL11.glGetInteger(37158, buffer);
                try {
                    Util.checkGLError();
                    profileMask = buffer.get(0);
                }
                catch (OpenGLException e2) {
                    LWJGLUtil.log("Failed to retrieve CONTEXT_PROFILE_MASK");
                }
            }
        }
        return profileMask;
    }
    
    static void initNativeStubs(final Class extension_class, final Set supported_extensions, final String ext_name) {
        resetNativeStubs(extension_class);
        if (supported_extensions.contains(ext_name)) {
            try {
                AccessController.doPrivileged((PrivilegedExceptionAction<Object>)new PrivilegedExceptionAction() {
                    public Object run() throws Exception {
                        final Method init_stubs_method = extension_class.getDeclaredMethod("initNativeStubs", (Class[])null);
                        init_stubs_method.invoke(null, (Object[])null);
                        return null;
                    }
                });
            }
            catch (Exception e) {
                LWJGLUtil.log("Failed to initialize extension " + extension_class + " - exception: " + e);
                supported_extensions.remove(ext_name);
            }
        }
    }
    
    public static synchronized void useContext(final Object context) throws LWJGLException {
        useContext(context, false);
    }
    
    public static synchronized void useContext(final Object context, final boolean forwardCompatible) throws LWJGLException {
        if (context == null) {
            ContextCapabilities.unloadAllStubs();
            setCapabilities(null);
            if (GLContext.did_auto_load) {
                unloadOpenGLLibrary();
            }
            return;
        }
        if (GLContext.gl_ref_count == 0) {
            loadOpenGLLibrary();
            GLContext.did_auto_load = true;
        }
        try {
            final ContextCapabilities capabilities = GLContext.capability_cache.get(context);
            if (capabilities == null) {
                new ContextCapabilities(forwardCompatible);
                GLContext.capability_cache.put(context, getCapabilities());
            }
            else {
                setCapabilities(capabilities);
            }
        }
        catch (LWJGLException e) {
            if (GLContext.did_auto_load) {
                unloadOpenGLLibrary();
            }
            throw e;
        }
    }
    
    public static synchronized void loadOpenGLLibrary() throws LWJGLException {
        if (GLContext.gl_ref_count == 0) {
            nLoadOpenGLLibrary();
        }
        ++GLContext.gl_ref_count;
    }
    
    private static native void nLoadOpenGLLibrary() throws LWJGLException;
    
    public static synchronized void unloadOpenGLLibrary() {
        --GLContext.gl_ref_count;
        if (GLContext.gl_ref_count == 0 && LWJGLUtil.getPlatform() != 1) {
            nUnloadOpenGLLibrary();
        }
    }
    
    private static native void nUnloadOpenGLLibrary();
    
    static native void resetNativeStubs(final Class p0);
    
    static {
        current_capabilities = new ThreadLocal();
        GLContext.fast_path_cache = new CapabilitiesCacheEntry();
        thread_cache_entries = new ThreadLocal();
        capability_cache = new WeakHashMap();
        Sys.initialize();
    }
    
    private static final class CapabilitiesCacheEntry
    {
        Thread owner;
        ContextCapabilities capabilities;
    }
}
