package org.lwjgl.opengl;

import org.lwjgl.*;
import java.nio.*;

final class WindowsKeyboard
{
    private static final int MAPVK_VK_TO_VSC = 0;
    private static final int BUFFER_SIZE = 50;
    private final long hwnd;
    private final ByteBuffer keyboard_state;
    private final byte[] key_down_buffer;
    private final EventQueue event_queue;
    private final ByteBuffer tmp_event;
    private boolean grabbed;
    private boolean has_retained_event;
    private int retained_key_code;
    private byte retained_state;
    private int retained_char;
    private long retained_millis;
    private boolean retained_repeat;
    
    WindowsKeyboard(final long hwnd) throws LWJGLException {
        this.key_down_buffer = new byte[256];
        this.event_queue = new EventQueue(18);
        this.tmp_event = ByteBuffer.allocate(18);
        this.hwnd = hwnd;
        this.keyboard_state = BufferUtils.createByteBuffer(256);
    }
    
    private static native boolean isWindowsNT();
    
    public void destroy() {
    }
    
    boolean isKeyDown(final int lwjgl_keycode) {
        return this.key_down_buffer[lwjgl_keycode] == 1;
    }
    
    public void grab(final boolean grab) {
        if (grab) {
            if (!this.grabbed) {
                this.grabbed = true;
            }
        }
        else if (this.grabbed) {
            this.grabbed = false;
        }
    }
    
    public void poll(final ByteBuffer keyDownBuffer) {
        final int old_position = keyDownBuffer.position();
        keyDownBuffer.put(this.key_down_buffer);
        keyDownBuffer.position(old_position);
    }
    
    private static native int MapVirtualKey(final int p0, final int p1);
    
    private static native int ToUnicode(final int p0, final int p1, final ByteBuffer p2, final CharBuffer p3, final int p4, final int p5);
    
    private static native int ToAscii(final int p0, final int p1, final ByteBuffer p2, final ByteBuffer p3, final int p4);
    
    private static native int GetKeyboardState(final ByteBuffer p0);
    
    private static native int GetKeyState(final int p0);
    
    private void putEvent(final int keycode, final byte state, final int ch, final long millis, final boolean repeat) {
        this.tmp_event.clear();
        this.tmp_event.putInt(keycode).put(state).putInt(ch).putLong(millis * 1000000L).put((byte)(repeat ? 1 : 0));
        this.tmp_event.flip();
        this.event_queue.putEvent(this.tmp_event);
    }
    
    private boolean checkShiftKey(final int virt_key, final byte state) {
        final int key_state = GetKeyState(virt_key) >>> 15 & 0x1;
        final int lwjgl_code = WindowsKeycodes.mapVirtualKeyToLWJGLCode(virt_key);
        return this.key_down_buffer[lwjgl_code] == 1 - state && key_state == state;
    }
    
    private int translateShift(final int scan_code, final byte state) {
        if (this.checkShiftKey(160, state)) {
            return 160;
        }
        if (this.checkShiftKey(161, state)) {
            return 161;
        }
        if (scan_code == 42) {
            return 160;
        }
        if (scan_code == 54) {
            return 161;
        }
        return 160;
    }
    
    private int translateExtended(final int virt_key, final int scan_code, final byte state, final boolean extended) {
        switch (virt_key) {
            case 16: {
                return this.translateShift(scan_code, state);
            }
            case 17: {
                return extended ? 163 : 162;
            }
            case 18: {
                return extended ? 165 : 164;
            }
            default: {
                return virt_key;
            }
        }
    }
    
    private void flushRetained() {
        if (this.has_retained_event) {
            this.has_retained_event = false;
            this.putEvent(this.retained_key_code, this.retained_state, this.retained_char, this.retained_millis, this.retained_repeat);
        }
    }
    
    public void handleKey(int virt_key, final int scan_code, final boolean extended, final byte event_state, final long millis, final boolean repeat) {
        virt_key = this.translateExtended(virt_key, scan_code, event_state, extended);
        this.flushRetained();
        this.has_retained_event = true;
        final int keycode = WindowsKeycodes.mapVirtualKeyToLWJGLCode(virt_key);
        if (keycode < this.key_down_buffer.length) {
            this.key_down_buffer[keycode] = event_state;
        }
        this.retained_key_code = keycode;
        this.retained_state = event_state;
        this.retained_millis = millis;
        this.retained_char = 0;
        this.retained_repeat = repeat;
    }
    
    public void handleChar(final int event_char, final long millis, final boolean repeat) {
        if (this.has_retained_event && this.retained_char != 0) {
            this.flushRetained();
        }
        if (!this.has_retained_event) {
            this.putEvent(0, (byte)0, event_char, millis, repeat);
        }
        else {
            this.retained_char = event_char;
        }
    }
    
    public void read(final ByteBuffer buffer) {
        this.flushRetained();
        this.event_queue.copyEvents(buffer);
    }
}
